var dbLib = require('../../lib/db')
var request = require('request')
var inspect = require('eyespect').inspector()
var should = require('should');
var assert = require('assert');
var fs = require('fs');
var path = require('path');
var configFilePath = path.join(__dirname, '..', 'config.json');
assert.ok(fs.existsSync(configFilePath) ,'config file not found at path: ' + configFilePath);
var config = require('nconf').argv().env().file({file: configFilePath});
var index = require('../../index')
var createTestUsers = require('create-test-users')
describe('Spinup Server', function () {
  this.timeout('20s')
  this.slow('10s')
  var online = false,
      port,
      usersJSON,
      role = 'parseTest'

  var desiredMessage = 'dummy parse called'

  before(function (done) {
    dbLib(config, function (err, dbReply) {
      should.not.exist(err)
      should.exist(dbReply)
      usersJSON = JSON.parse(fs.readFileSync(path.join(__dirname, 'users.json'), 'utf8'))
      should.exist(usersJSON, 'failed to read test users json')
      var testUsersData = {
        db: dbReply,
        usersData: usersJSON,
        config: config,
        confirmPasswordsMatch: false,
        remove: false
      }
      createTestUsers(testUsersData, function (err, reply) {
        should.not.exist(err, 'error creating users: '+ JSON.stringify(err, null, ' '))
        done()
      });
    })
  })
  it('should spinup', function (done) {
    var parse = function (req, res) {
      var id = req.params.id
      inspect('dummy parse called')
      var output = {
        message: desiredMessage,
        id: id
      }
      res.send(output)
    }
    var data = {
      config: config,
      parse: parse,
      role: role
    }
    index.rebuildAndStart(data, function (err, reply) {
      should.not.exist(err)
      should.exist(reply)
      should.exist(reply.port, 'port field missing from reply')
      should.exist(reply.server, 'server field missing from reply')
      online = true
      port = reply.port
      done()
    })
  })

  it('should reject anonymous posts to parse url', function (done) {
    assert.ok(online, 'the spinup test must pass before this one can run')
    should.exist(port, 'port should be set')
    var id = 'fooID'
    var url = 'http://localhost:' + port + '/api/' + role + '/parse/' + id
    request.post(url, function (err, res, body) {
      should.not.exist(err, 'err posting to url: ' + JSON.stringify(err, null, ' '))
      res.statusCode.should.eql(401)
      body.should.eql('Unauthorized')
      done()
    })
  })

  it('should accept authenticated posts to parse url', function (done) {
    assert.ok(online, 'the spinup test must pass before this one can run')
    should.exist(port, 'port should be set')
    var userData = usersJSON[0]
    should.exist(userData, 'failed to get user data')
    var email = encodeURIComponent(userData.email)
    var password = encodeURIComponent(userData.password)
    should.exist(email, 'failed to get user data email')
    should.exist(password, 'failed to get user data password')
    var id = 'fooID'
    var url = 'http://' + email + ':' + password + '@localhost:' + port
          + '/api/' + role + '/parse/' + id
    request.post(url, function (err, res, body) {
      should.not.exist(err, 'err posting to url: ' + JSON.stringify(err, null, ' '))
      var resData = JSON.parse(body)
      resData.message.should.eql(desiredMessage)
      resData.id.should.eql(id)
      res.statusCode.should.eql(200)
      done()
    })
  })
})
